%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% MC study to consider nonparametric estimation of EA and SP auctions

clear all
clc
tic

% switch to turn on plotting: yesplot = 1, then plot, yesplot = 0, no plot
yesplot = 0;
% switch to turn on printing: yesprint = 1, then print output to file,
% yesprint = 0, no printing to file
yesprint = 0;

% set seed
rand('state',81981)

kernel = 'triweight';
% b0 is needed for h0 bandwidth---see KZ (2008) bottom of page 501 and
% the K0 term in (4.9) of ZK (1998) which is K0 = 6 + 18t + 12t^2,
% t\in[-1,0]; this will compute b0 for any kernel choice provided by
% user
num_t1 = quad(@evalkspdf_num,-1,1,1e-12,[],kernel);
num_t2 = 0 - (-144/5 + 432/4 - 468/3 + 216/2 - 36);
denom_t1 = 0 - (-2 + 18/4 - 12/5);
denom_t2 = quad(@evalkspdf_denom,-1,1,1e-12,[],kernel);
b0 = ((num_t1^2*num_t2)/(denom_t1^2*denom_t2))^(1/5);

% number of simulations, auctions per simulation, and bidders at auction
S = 1000;

% distribution of interest
vlow = 0;
vhigh = 1;
npts = 1000;
vgrid = linspace(vlow,vhigh,npts);

delta = 0.02;

%matlabpool open local
%Distributions = {'power','exponential','Rayleigh'};
Distributions = {'power'};
%Parameters    = [1.5, 1.5; 2, 1; .3, .5;]; %Indexed by Dindx
Parameters    = [1.5, 1.5;]; %Indexed by Dindx
Players       = [3; 5; 10];  % Indexed by Pindx
SampleSize    = [100, 300];  %Indexed by Sindx

allb = NaN*ones(300*10,1);
allv = NaN*ones(300*10,1);
allv_ea = NaN*ones(300*10,1);
allv_spa = NaN*ones(300*10,1);
for dindx=1:length(Distributions)
	toc
    for pindx=1:length(Players)
        for sindx=1:length(SampleSize)
			display(strcat('CURRENTLY WORKING ON CASE: ',num2str(dindx),'-',num2str(pindx),'-',num2str(sindx)))
            rand('state',81981)
            % number of auctions per simulation and bidders at auction
            T = SampleSize(sindx);
            N = Players(pindx);
            
            
            fbc_ture = NaN*ones(S,npts);
            fbc_spa = NaN*ones(S,npts);
            fbc_ea = NaN*ones(S,npts);
            Fbc_true = NaN*ones(S,npts);
            Fbc_spa = NaN*ones(S,npts);
            Fbc_ea = NaN*zeros(S,npts);
            HoKS_ea_true = NaN*ones(S,1);
            pKS_ea_true = NaN*ones(S,1);
            KS_ea_true = NaN*ones(S,1);
            HoKS_spa_true = NaN*ones(S,1);
            pKS_spa_true = NaN*ones(S,1);
            KS_spa_true = NaN*ones(S,1);
            HoKS_ea_spa = NaN*ones(S,1);
            pKS_ea_spa = NaN*ones(S,1);
            KS_ea_spa = NaN*ones(S,1);
            HoAD_ea_true = NaN*ones(S,1);
            pAD_ea_true = NaN*ones(S,1);
            AD_ea_true = NaN*ones(S,1);
            HoAD_spa_true = NaN*ones(S,1);
            pAD_spa_true = NaN*ones(S,1);
            AD_spa_true = NaN*ones(S,1);
            HoAD_ea_spa = NaN*ones(S,1);
            pAD_ea_spa = NaN*ones(S,1);
            AD_ea_spa = NaN*ones(S,1);
            expRspaN = NaN*ones(S,1);
            expRspaadd = NaN*ones(S,1);
            expReaN = NaN*ones(S,1);
            expReaadd = NaN*ones(S,1);
            expRtrueN = NaN*ones(S,1);
            expRtrueadd = NaN*ones(S,1);
            fzero_s = NaN*ones(S,1);
            fzero_spa = NaN*ones(S,1);
            fzero_ea = NaN*ones(S,1);
            rhoflag = NaN*ones(S,1);
            rhoflag_spa = NaN*ones(S,1);
            rhoflag_ea = NaN*ones(S,1);
            rho_s = NaN*ones(S,1);
            rho_spa = NaN*ones(S,1);
            rho_ea = NaN*ones(S,1);
            
            vals = [];
            bids = [];
            rho_true = [];
            u = [];
            v = [];
            b = [];
            v_spa = [];
            gb = [];
            Gb = [];
            hg_ea = [];
            Gb_delta = [];
            v_ea = [];
            X_ea = [];
            X_spa = [];
            X_ea_spa = [];
            randinds= [];
            justNinds = [];
            truevalsN = [];
            truetwoN = [];
            truevalsadd = [];
            truetwoadd = [];
            spavalsN = [];
            spatwoN = [];
            spavalsadd = [];
            spatwoadd = [];
            eavalsN = [];
            eatwoN = [];
            eavalsadd = [];
            eatwoadd = [];
        

	
			% if exponential think of param1 = lambda (probably want to be
            % greater than 1...although mean is different given
            % truncation); if rayleigh probably want mean to be less than
            % sqrt(pi)/2 so that mean is less than one. weibull is an
            % exponential if param2 = 1 and is a rayleigh if param2 = 2. 
            if dindx==1
                dist = 'power';
            elseif dindx == 2
                dist = 'exponential';
            elseif dindx==3
                dist = 'rayleigh';
            elseif dindx==4
                dist = 'weibull';
            end
            param1 = Parameters(dindx,1);
            param2 = Parameters(dindx,2);
			pvect = param1;
			if strcmpi(dist,'weibull')
				pvect = [pvect param2];
            end
            
            % solve for the inverse-bid function given the distribution
            rkorder = 4;
            [vals,bids] = EAauction(N,vhigh,delta,dist,param1,param2,rkorder);
            if sum(vals - bids < 0) > 0
                warning('Bid function is not rational: some bids exceed valuations\n')
            end
            
            % true optimal reserve price
            rhoguess = (vhigh + vlow)/2;
            rho_true = fzero('optreserve',rhoguess,[],dist,vhigh,pvect);

%parfor s=1:S
for s=1:S
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 1: get randomly drawn valuations and bids given distribution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% generate valuations from the distribution of interest via inverse cdf
u = rand(N*T,1);
% sorted for convenience when plotting
u = sort(u);
v = inversecdf(u,dist,param1,param2,vlow,vhigh);
allv(1:length(v),size(allv,2)+1) = v;

% get EA inverse-bid function evaluated at the randomly-drawn valuations so
% that we have randomly-drawn bids
b = spline(vals,bids,v);
allb(1:length(v),size(allv,2)+1) = b;
if sum(v - b < 0) > 0
	warning('Bid function is not rational: some bids exceed valuations\n')
end
if (mean(v) > vhigh/2) && strcmpi(dist,'exponential')
	warning('MLE estimator will not exist for this data\n')
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 2: estimate nonparametrically for SP case
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% in spa, do gpv on bids directly as b = v
v_spa = b;
allv_spa(1:length(v),size(allv,2)+1) = v_spa;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 3: estimate nonparametrically for EA case
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% in ea, do gpv on correctly-specified pseudo-values
[gb,hg_ea] = kspdf_bc(b,kernel,b,b0);
Gb = kscdf_new(b,'edf',b);
Gb_delta = kscdf_new(b,'edf',b-delta);
v_ea = b' + (Gb.^(N - 1) - Gb_delta.^(N - 1))./((N - 1).*gb.*Gb.^(N - 2));
v_ea = v_ea';
allv_ea(1:length(v),size(allv,2)+1) = v_ea;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 4: some nonparametric tests
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% two-sample K--S statistic comparing EA or SPA EDFs to EDF of sample of
% random valuations
[HoKS_ea_true(s),pKS_ea_true(s),KS_ea_true(s)] = kstest2(v_ea,v);
[HoKS_spa_true(s),pKS_spa_true(s),KS_spa_true(s)] = kstest2(v_spa,v);
[HoKS_ea_spa(s),pKS_ea_spa(s),KS_ea_spa(s)] = kstest2(v_spa,v_ea);

% two-sample Anderson--Darling test to see if distributions that data were
% drawn from are statistically identical---if Ho == 0 then fail to reject
% that they come from same dist; if Ho = 1 then reject that they come from
% same dist.
X_ea = [v_ea ones(size(v_ea)); v 2*ones(size(v))];
[AD_ea_true(s),pAD_ea_true(s),HoAD_ea_true(s)] = AnDarksamtest(X_ea);
X_spa = [v_spa ones(size(v_spa)); v 2*ones(size(v))];
[AD_spa_true(s),pAD_spa_true(s),HoAD_spa_true(s)] = AnDarksamtest(X_spa);
X_ea_spa = [v_spa ones(size(v_spa)); v_ea 2*ones(size(v_ea))];
[AD_ea_spa(s),pAD_ea_spa(s),HoAD_ea_spa(s)] = AnDarksamtest(X_ea_spa);

if (yesplot == 2)
    [fbc_true,h_true] = kspdf_bc(v,kernel,vgrid,b0);
    [fbc_spa,h_spa] = kspdf_bc(v_spa,kernel,vgrid,b0);
    [fbc_ea,hf_ea] = kspdf_bc(v_ea,kernel,vgrid,b0);
    figure
    plot(vgrid,fbc_true,vgrid,fbc_spa,vgrid,fbc_ea)
    legend('f_s','f_{spa}','f_{ea}','Location','South')
    
    F_true = kscdf_new(v,'edf',vgrid);
    F_spa = kscdf_new(v_spa,'edf',vgrid);
    F_ea = kscdf_new(v_ea,'edf',vgrid);
    figure
    plot(vgrid,F_true,vgrid,F_spa,vgrid,F_ea)
    legend('F_s','F_{spa}','F_{ea}','Location','NorthWest')
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 5: compute implied optimal reserve prices
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fzeropt = optimset('TolX',1e-16);
[rho_s(s),fzero_s(s),rhoflag(s)] = ...
    fminbnd('optreserve_np',.1,.9,fzeropt,v,b0,kernel);
[rho_spa(s),fzero_spa(s),rhoflag_spa(s)] = ...
    fminbnd('optreserve_np',.1,.9,fzeropt,v_spa,b0,kernel);
[rho_ea(s),fzero_ea(s),rhoflag_ea(s)] = ...
    fminbnd('optreserve_np',.1,.9,fzeropt,v_ea,b0,kernel);
if (rhoflag(s) < 1) || (rhoflag_spa(s) < 1) || ...
        (rhoflag_ea(s) < 1)
    fprintf('No convergence in finding optimal reserve price at itearation %d\n',s)
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 6: see effects of adding another bidder to auction
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% draw random uniform integers from 1 to NT
randinds = ceil((N*T)*rand(N+1,S));
justNinds = randinds(1:N,:);

% sample from SPA valuations
spavalsN = sort(v_spa(justNinds));
spatwoN = spavalsN(N-1,:);
expRspaN(s) = mean(spatwoN);
spavalsadd = sort(v_spa(randinds));
spatwoadd = spavalsadd(N,:);
expRspaadd(s) = mean(spatwoadd);
% sample from EA valuations
eavalsN = sort(v_ea(justNinds));
eatwoN = eavalsN(N-1,:);
expReaN(s) = mean(eatwoN);
eavalsadd = sort(v_ea(randinds));
eatwoadd = eavalsadd(N,:);
expReaadd(s) = mean(eatwoadd);
% sample from true valuations
truevalsN = sort(v(justNinds));
truetwoN = truevalsN(N-1,:);
expRtrueN(s) = mean(truetwoN);
truevalsadd = sort(v(randinds));
truetwoadd = truevalsadd(N,:);
expRtrueadd(s) = mean(truetwoadd);


end


% difference in expected revenues from adding another bidders
diffspaadd = expRspaadd - expRspaN;
diffeaadd = expReaadd - expReaN;
difftrueadd = expRtrueadd - expRtrueN;
dspatrue = expRspaadd - expRtrueN;
deatrue = expReaadd - expRtrueN;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 7: write output to file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (yesprint == 1)
    fname1 = strcat('np_mcreserve.out');
    fid = fopen(fname1,'a');
    if dindx == 1 && pindx == 1 && sindx == 1
        fprintf(fid,'********  NEW RUN  delta = %.2f ********\n',delta);
    end 
    fprintf(fid,'%s(%.2f,%.2f) \n',dist,param1,param2);
    fprintf(fid,'& $%d$ & $%d$ & $%.5f$ & $%.5f$ & $%.5f$ \\\\\n', ...
        T,N,mean(rho_s),mean(rho_ea),mean(rho_spa));
    fprintf(fid,'& & & $(%.5f)$ & $(%.5f)$ & $(%.5f)$ \\\\\n', ...
        std(rho_s),std(rho_ea),std(rho_spa));
    fclose(fid);
    
    fname2 = strcat('np_ks.out');
    fid = fopen(fname2,'a');
    if dindx == 1 && pindx == 1 && sindx == 1
        fprintf(fid,'********  NEW RUN  delta = %.2f ********\n',delta);
    end
    fprintf(fid,'%s(%.2f,%.2f)\n',dist,param1,param2);
    fprintf(fid,'& $%d$ & $%d$ & $%.5f$ & $%.5f$ & $%.5f$ \\\\\n', ...
        T,N,mean(pKS_ea_true),mean(pKS_spa_true),mean(pKS_ea_spa));
    fprintf(fid,'& $%d$ & $%d$ & $%.5f$ & $%.5f$ & $%.5f$ \\\\\n', ...
        T,N,median(pKS_ea_true),median(pKS_spa_true),median(pKS_ea_spa));
    fprintf(fid,'& $%d$ & $%d$ & $(%.5f)$ & $(%.5f)$ & $(%.5f)$ \\\\\n', ...
        T,N,std(pKS_ea_true),std(pKS_spa_true),std(pKS_ea_spa));
    fprintf(fid,'& & & $%d$ & $%d$ & $%d$ \\\\\n', ...
        sum(HoKS_ea_true),sum(HoKS_spa_true),sum(HoKS_ea_spa));
    fclose(fid);
    
    fname3 = strcat('np_ad.out');
    fid = fopen(fname3,'a');
    if dindx == 1 && pindx == 1 && sindx == 1
        fprintf(fid,'********  NEW RUN  delta = %.2f ********\n',delta);
    end
    fprintf(fid,'%s(%.2f,%.2f)\n',dist,param1,param2);
    fprintf(fid,'& $%d$ & $%d$ & $%.5f$ & $%.5f$ & $%.5f$ \\\\\n', ...
        T,N,mean(pAD_ea_true),mean(pAD_spa_true),mean(pAD_ea_spa));
    fprintf(fid,'& $%d$ & $%d$ & $%.5f$ & $%.5f$ & $%.5f$ \\\\\n', ...
        T,N,median(pAD_ea_true),median(pAD_spa_true),median(pAD_ea_spa));
    fprintf(fid,'& $%d$ & $%d$ & $(%.5f)$ & $(%.5f)$ & $(%.5f)$ \\\\\n', ...
        T,N,std(pAD_ea_true),std(pAD_spa_true),std(pAD_ea_spa));
    fprintf(fid,'& & & $%d$ & $%d$ & $%d$ \\\\\n', ...
        sum(HoAD_ea_true),sum(HoAD_spa_true),sum(HoAD_ea_spa));
    fclose(fid);
    
    fname4 = strcat('np_addbidder.out');
    fid = fopen(fname4,'a');
    if dindx == 1 && pindx == 1 && sindx == 1
        fprintf(fid,'********  NEW RUN  delta = %.2f ********\n',delta);
    end
    fprintf(fid,'%s(%.2f,%.2f) \n',dist,param1,param2);
    fprintf(fid,'& $%d$ & $%d$ & $%.5f$ & $%.5f$ & $%.5f$ \\\\\n', ...
        T,N,mean(expRtrueadd),mean(expReaadd),mean(expRspaadd));
    fprintf(fid,'& & & $(%.5f)$ & $(%.5f)$ & $(%.5f)$ \\\\\n', ...
        std(expRtrueadd),std(expReaadd),std(expRspaadd));
    fclose(fid);
    
    fname5 = strcat('np_Nbidder.out');
    fid = fopen(fname5,'a');
    if dindx == 1 && pindx == 1 && sindx == 1
        fprintf(fid,'********  NEW RUN  delta = %.2f ********\n',delta);
    end
    fprintf(fid,'%s(%.2f,%.2f) \n',dist,param1,param2);
    fprintf(fid,'& $%d$ & $%d$ & $%.5f$ & $%.5f$ & $%.5f$ \\\\\n', ...
        T,N,mean(expRtrueN),mean(expReaN),mean(expRspaN));
    fprintf(fid,'& & & $(%.5f)$ & $(%.5f)$ & $(%.5f)$ \\\\\n', ...
        std(expRtrueN),std(expReaN),std(expRspaN));
    fclose(fid);
    
    fname6 = strcat('np_adddiff.out');
    fid = fopen(fname6,'a');
    if dindx == 1 && pindx == 1 && sindx == 1
        fprintf(fid,'********  NEW RUN  delta = %.2f ********\n',delta);
    end
    fprintf(fid,'%s(%.2f,%.2f) \n',dist,param1,param2);
    fprintf(fid,'& $%d$ & $%d$ & $%.5f$ & $%.5f$ & $%.5f$ & $%.5f$ & $%.5f$ \\\\\n', ...
        T,N,mean(difftrueadd),mean(diffeaadd),mean(diffspaadd), ...
        mean(dspatrue),mean(deatrue));
    fprintf(fid,'& & & $(%.5f)$ & $(%.5f)$ & $(%.5f)$ & $(%.5f)$ & $(%.5f)$ \\\\\n', ...
        std(difftrueadd),std(diffeaadd),std(diffspaadd), ...
        std(dspatrue),std(deatrue));
    fclose(fid);
    
end

        end
    end
end

%matlabpool close

toc