function [lambda,lsqobjfnval,exitflag,output] =... 
    FilterOptNaive(obsdata,lambda0,output,solver)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
%%%%%%%%%%%%  IMPORTANT NOTE:  the notation in the code here regarding N vs
%%%%%%%%%%%%  Ntilde is the reverse of the new version of the paper as of
%%%%%%%%%%%%  September 2015!  The paper denotes observed bidders as N
%%%%%%%%%%%%  and total potential bidders (but unobserved) as Ntilde.  HERE
%%%%%%%%%%%%  IN THE CODE, THE REVERSE IS TRUE, reflecting a notation
%%%%%%%%%%%%  convention which pre-dates the current revision of the paper:
%%%%%%%%%%%%  in this program Ntilde is the (unobserved) total
%%%%%%%%%%%%  participation and N is the observed number of bidders.

if nargin<4
    solver = 'knitro';
end
mu     = mean(obsdata(:,1));
sigma2 = var(obsdata(:,1));
A    = [];  
b    = [];  
Aeq  = [];  
beq  = [];
lb   = [2; -0.99];
ub   = [100; 0.99];
nonlcon = [];


xtolerance   = 1e-8;
funtolerance = 1e-8;
constrainttolerance = 1e-8;
constraintgradient = 'off';
disp = 'off';

if output==0 | strcmpi(output,'off')
    if strcmpi(solver,'knitro')
        opts = optimoptions('ktrlink','GradObj',constraintgradient,...
                            'tolx',xtolerance,'tolfun',funtolerance,...
                            'tolcon',constrainttolerance);
    elseif strcmpi(solver,'fmincon')
        opts = optimoptions('fmincon','UseParallel','always',...
                            'Algorithm','active-set','GradObj',constraintgradient,...
                            'tolx',xtolerance,'tolfun',funtolerance,...
                            'tolcon',constrainttolerance);
    end
else
    if strcmpi(solver,'knitro')
        opts = optimoptions('ktrlink','Display',disp,...
                            'GradObj',constraintgradient,'tolx',xtolerance,...
                            'tolfun',funtolerance,'tolcon',constrainttolerance);
    elseif strcmpi(solver,'fmincon')
        opts = optimoptions('fmincon','UseParallel','always',...
                            'Algorithm','active-set','Display',disp,...
                            'GradObj',constraintgradient,'tolx',xtolerance,...
                            'tolfun',funtolerance,'tolcon',constrainttolerance);
    end
end

if strcmpi(solver,'knitro')
    [lambda,lsqobjfnval] = ktrlink(@nestedfun,lambda0,A,b,Aeq,beq,lb,ub,nonlcon,opts);
elseif strcmpi(solver,'fmincon')
    [lambda,lsqobjfnval,exitflag,output] = fmincon(@nestedfun,lambda0,A,b,Aeq,beq,lb,ub,nonlcon,opts);
end


    function [lsqobjfn,grad] = nestedfun(lambda)
        grad = [];
        
        lsqobjfn  = (lambda(1)/(1-lambda(2)) - mu)^2 + (lambda(1)/(1-lambda(2))^3 - sigma2)^2;
    end



end

