function [betaB] = GalerkinSolveEA(lambda,beta,knotvecV,knotvecB,delta1,delta2,bstar)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Code by: Brent Hickman, University of Chicago                     %%%%
%%%% Last updated: January 2016                                        %%%%
%%%% Email for comments/bugs: hickmanbr@uchicago.edu                   %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% bdom = sort([knotvecB; (knotvecB(1)+knotvecB(2))/2; (knotvecB(end-1)+knotvecB(end))/2]);
bdom = [knotvecB(1); (3/4)*knotvecB(1)+(1/4)*knotvecB(2); (1/2)*(knotvecB(1:end-1)+knotvecB(2:end)); knotvecB(end)];
[BasisCheckB,~,DerivCheckB] = BsplineBasis3(knotvecB,bdom,'off');
bdomdelta = [(bdom(bdom<bstar+delta1)-delta1);...
              bstar*ones(sum(bdom>=bstar+delta1 & bdom<bstar+delta2),1);...
             (bdom(bdom>=bstar+delta2)-delta2)];
bdomdelta(bdomdelta<knotvecB(1)) = knotvecB(1); 
BasisCheckDelta = BsplineBasis3(knotvecB,bdomdelta,'off');

K  = length(knotvecV) - 1;
J = length(knotvecB) - 1;
LB = length(BasisCheckB(:,1));

betaVPE = beta(1:K+3);
betaBPE = beta(K+4:K+3+J+3);

options = optimoptions('fsolve','MaxIter',1000,'MaxFunEvals',100000,'Display','iter');
betaB = fsolve(@NLSystem,betaBPE,options);

    function [EAEquilibResid] = NLSystem(betaB)
        xi        = BasisCheckB*betaB;
        xiDelta   = BasisCheckDelta*betaB;
        xiprime   = DerivCheckB*betaB;
        [BasisV,~,DerivV] = BsplineBasis3(knotvecV,xi,'off');
        BasisVDelta       = BsplineBasis3(knotvecV,xiDelta,'off');
        FV        = BasisV*betaVPE;
        FVDelta   = BasisVDelta*betaVPE;
        fV        = DerivV*betaVPE;
        GB        = FV;
        GBDelta   = FVDelta;
        gB        = fV.*xiprime;
        %%%%Here we compute the Distribution and density functions for the
        %%%%highest competing bid:
        maxN      = 100;
        N         = 1:1:maxN;
        probN     = poissgenpdf(N,lambda);
        normconst = 1/(1 - poissgencdf(1,lambda));
        weight    = probN.*normconst;
        GM        = zeros(size(GB));
        GMDelta   = zeros(size(GBDelta));
        gM        = zeros(size(gB));
        for n=2:maxN
            GM = GM + weight(n)*GB.^(n - 1);
            GMDelta = GMDelta + weight(n)*GBDelta.^(n - 1);
            gM = gM + weight(n)*(n - 1).*GB.^(n - 2).*gB;
        end
        %%%%The next line contains the vector of nonlinear constraints.
        %%%%They say that, for any value of the parameter vector, the
        %%%%monotone hazard rate condition must be satisfied at each of the
        %%%%points in DerivCheckPts:
        indx1 = find(bdom<=knotvecB(1)+delta1,1,'last');
        indx2 = find(bdom<=bstar+delta1,1,'last');
        indx3 = find(bdom<=bstar+delta2,1,'last');
        indx4 = length(bdom);
		R0 = [(xi(1)-knotvecV(1)); (xi(2:indx1)-bdom(2:indx1)).*gM(2:indx1) - GM(2:indx1)];
        R1 = (xi(indx1+1:indx2)-bdom(indx1+1:indx2)).*gM(indx1+1:indx2)... 
                 - GM(indx1+1:indx2) + GMDelta(indx1+1:indx2);
        R2 = (xi(indx2+1:indx3)-bdom(indx2+1:indx3)).*gM(indx2+1:indx3)... 
                 - GM(indx2+1:indx3) + GMDelta(indx2+1:indx3);
        R3 = (xi(indx3+1:indx4)-bdom(indx3+1:indx4)).*gM(indx3+1:indx4) - GM(indx3+1:indx4) + GMDelta(indx3+1:indx4);
             
        EAEquilibResid = [R0; R1; R2; R3];
    end
end