function [fhat,h] = kspdf_bc(obs,kernel,evalgrid,b0,minasym,maxasym)
%  Brent R. Hickman and Timothy P. Hubbard
% hickmanbr@uchicago.edu and timothy.hubbard@colby.edu
%
% please cite our Journal of Applied Econometrics paper "Replacing Sample
% Trimming with Boundary Correction in Nonparametric Estimation of
% First-Price Auctions" if this code is helpful to you.
%
% compute boundary-corrected kernel smoothed estimate of pdf

% number of points in observed data
n = length(obs);

if (exist('minasym','var'))
    [k,h] = kzroutine(obs,evalgrid,kernel,b0,minasym);
    kright = kzroutine(-obs,-evalgrid,kernel,b0,-maxasym);
    % replace values near right boundary with right-corrected values
    rightindy = find(obs >= max(obs) - h);
    k(rightindy) = kright(rightindy);
else
    [k,h] = kzroutine(obs,evalgrid,kernel,b0);
    kright = kzroutine(-obs,-evalgrid,kernel,b0);
    % replace values near right boundary with right-corrected values
    rightindy = find(obs >= max(obs) - h);
    k(rightindy) = kright(rightindy);
end

% k is structured so #rows = size(evalgrid) while #cols = size(obs) so
% adjust and output kernel estimate
k = k';
fhat = sum(k)./(n*h);
