function v = inversecdf(u,dist,a,b,vlow,vhigh)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% gerneate random draws from some truncated distributions


if strcmpi(dist,'lognormal')
elseif strcmpi(dist,'normal')
elseif strcmpi(dist,'rayleigh')
	Fhigh = 1 - exp(-vhigh^2/(2*a^2));
	Flow = 1 - exp(-vlow^2/(2*a^2));
	logterm = 1 - Flow - u*(Fhigh - Flow);
	v = sqrt(-2*a^2*log(logterm));
elseif strcmpi(dist,'weibull')
	Fhigh = 1 - exp(-(vhigh/a)^b);
	Flow = 1 - exp(-(vlow/a)^b);
	logterm = 1 - Flow - u*(Fhigh - Flow);
	v = a*((-log(logterm)).^(1/b));
elseif strcmpi(dist,'exponential')
	Fhigh = 1 - exp(-a*vhigh);
	Flow = 1 - exp(-a*vlow);
	logterm = 1 - Flow - u*(Fhigh - Flow);
	v = -log(logterm)./a;
elseif strcmpi(dist,'power')
	v = u.^(1/a);
end

% some checks 
if isreal(v) == 0
	warning('Imaginary component of valuations')
	return
elseif sum(isinf(v)) > 0
	warning('Infinite valuations exist')
	return
elseif sum(isnan(v)) > 0
	warning('NaN valuations exist')
	return
end