%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% plot truncated PDFs for MC experiments

clear all
clc
% parameters for figure properties
lwidth = 2;
fsize = 14;
set(0,'defaulttextinterpreter','latex','Defaulttextfontsize',fsize);

vhigh = 1;
vlow = 0;
v = linspace(0,1,10000);

dist = 'weibull';
a = .5;
b = 4;
if strcmpi(dist,'weibull')
	F = 1 - exp(-(v./a).^b);
	f = (b/a).*(v./a).^(b - 1).*exp(-(v./a).^b);
	Ftr = 1 - exp(-(vhigh/a)^b);
	Fv_wbl = F./Ftr;
	fv_wbl = f./Ftr;
	lambda_wbl = fv_wbl./(1 - Fv_wbl);
end

dist = 'power';
a = 1.5;
if strcmpi(dist,'power')
	Fv_pwr = v.^a;
	fv_pwr = a*v.^(a - 1);
	lambda_pwr = fv_pwr./(1 - Fv_pwr);
end

dist = 'exponential';
a = 2;
if strcmpi(dist,'exponential')
	F = 1 - exp(-a*v);
	f = a*exp(-a*v);
	Ftr = 1 - exp(-a*vhigh);
	Fv_exp = F./Ftr;
	fv_exp = f./Ftr;
	lambda_exp = fv_exp./(1 - Fv_exp);
end

dist = 'rayleigh';
a = 0.3;
if strcmpi(dist,'rayleigh')
	F = 1 - exp(-(v.^2)./(2*a^2));
	f = v./a^2.*exp(-(v.^2)./(2*a^2));
	Ftr = 1 - exp(-(vhigh^2)./(2*a^2));
	Fv_ryl = F./Ftr;
	fv_ryl = f./Ftr;
	lambda_exp = fv_ryl./(1 - Fv_ryl);
end


figure1 = figure
set(gcf,'DefaultLineLineWidth',lwidth)
set(gca,'FontSize',fsize)
hold all
box on
plot(v,fv_pwr,v,fv_exp,v,fv_ryl)
xlim([0 1])
xlabel('$\mathbf{v}$')
ylabel('$\mathbf{f(v)}$')

annotation(figure1,'textbox',[0.1601 0.839 0.1538 0.06647],...
	'String',{'Exponential(2.0)'},...
	'Interpreter','latex',...
	'FontName','Times New Roman',...
	'FitBoxToText','off',...
	'LineStyle','none');
annotation(figure1,'textbox',[0.4541 0.7136 0.1538 0.06647],...
	'String',{'Rayleigh(0.3)'},...
	'Interpreter','latex',...
	'FontName','Times New Roman',...
	'FitBoxToText','off',...
	'LineStyle','none');
annotation(figure1,'textbox',[0.754 0.4736 0.1096 0.06647],...
	'String',{'Power(1.5)'},...
	'Interpreter','latex',...
	'FontName','Times New Roman',...
	'FitBoxToText','off',...
	'LineStyle','none');

print('-depsc','trpdf.eps')

figure1 = figure
set(gcf,'DefaultLineLineWidth',lwidth)
set(gca,'FontSize',fsize)
hold all
box on
plot(v,log(lambda_wbl),v,log(lambda_pwr),'-.',v,log(lambda_exp),'--')
xlim([0 1])
xlabel('$\mathbf{v}$')
ylabel('$\mathbf{\lambda(v)}$')


figure
set(gcf,'DefaultLineLineWidth',lwidth)
set(gca,'FontSize',fsize)
hold all
box on
plot(v,Fv_pwr,v,Fv_exp,v,Fv_ryl)
xlim([0 1])
xlabel('$\mathbf{v}$')
ylabel('$\mathbf{F(v)}$')

