function y = poissgencdf(n,lambda)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% define generalized Poisson cdf

if length(lambda) == 1
    lambda(1) = lambda;
    lambda(2) = 0;
end

if sum(n < 0) > 0
    error('poissgencdf:BadInputsN','poissgencdf is only defined for non-negative values of n.')
end
if ~(lambda(1) > 0) || ~(abs(lambda(2)) < 1)
    error('poissgencdf:BadInputsLAMBDA','poissgencdf is only defined for lambda(1)>0 and -1<=lambda(2)<1.')
end

y = poissgenpdf(0,lambda)*ones(size(n));
nmax = max(n);
ntemp = (1:1:nmax);
pdftemp = poissgenpdf(ntemp,lambda);

for i=1:length(n)
    if n(i) > 0
        y(i) = y(i) + sum(pdftemp(1:n(i)));
    end
end




