function y = poissgenpdf(n,lambda)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% define generalized Poisson pdf

if length(lambda) == 1
    lambda(1) = lambda;
    lambda(2) = 0;
end

if sum(n < 0) > 0
    error('poissgenpdf:BadInputsN','poissgenpdf is only defined for non-negative values of n.')
end
if ~(lambda(1) > 0) || ~(abs(lambda(2)) < 1)
    error('poissgenpdf:BadInputsLAMBDA','poissgenpdf is only defined for lambda(1)>0 and -1<=lambda(2)<1.')
end

y = zeros(size(n));
% the factorial function is only accurate up to 21, so this index tells us
% which values we can compute the easy way
indx = (n <= 21);

y(indx) = exp(-n(indx)*lambda(2)-lambda(1)).* ...
	(lambda(1)*(lambda(1) + n(indx)*lambda(2)).^(n(indx)-1))./ ...
	factorial(n(indx));

% now the index tells us which remaining values we have to compute the hard
% way
indx = find(indx==0);



for i=1:length(indx)
    l = indx(i);
    N = n(l);
    
    denom = (2:N)';
    num   = (lambda(1) + N*lambda(2))*ones(N-1,1);  
    % instead of doing an inaccurate factorial for large numbers, we take
    % the ratio of the (N-1) product terms in the numerator, with the (N-1)
    % nontrivial product terms in the factorial, and then we multiply the
    % ratios together.  finally, we finish it off by multiplying the
    % remaining terms to get y 
    y(l) = ( prod(num./denom) )*lambda(1)*exp(-N*lambda(2)-lambda(1));
end



