function y=pwrcdf(x,l,u,a)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
%PWRCDF(x,l,u,a,b) calculates the cdf of the power distribution, or the
%uniform distribution on [l,u] raised to the power of a, where a>0.  If a
%is left unspecified, its default value is 1, which results in a uniform
%distribution.
% 
%See also TLOGNCDF, TNORMCDF, TRAYLCDF, TLOGNPDF, TNORMPDF, TRAYLPDF, 
%TWBLPDF, TWBLCDF, PWRPDF

% make sure that bounds have been properly specified:
if nargin < 3
    error('YOU MUST AT LEAST SPECIFY AN INPUT AND LOWER/UPPER BOUNDS!');
end

if nargin < 4
    a = 1;
end

y = ( (x-l)/(u-l) ).^a;

% for functional values outside the truncated domain
y(find(x<=l))=0;
y(find(x>=u))=1;
