function [L,g,H] = rawllhood(p,v,dist,vhigh)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% define likelihood function for Monte Carlo study

a = p(1);
if length(p) == 2
	b = p(2);
else
	% value doesn't matter if one-parameter family
	b = 1;
end
Nobs = length(v);

if strcmpi(dist,'rayleigh')
	% log-likelihood function and negative since minimizing
	Fhigh = 1 - exp(-vhigh^2/(2*a^2));
	ehigh = exp(-vhigh^2/(2*a^2));
	L = sum(log(v) - 2*log(a) - v.^2./(2*a^2) - log(Fhigh));
	L = -L;
	% score vector and negative since minimizing
	g = -2*Nobs/a + sum(v.^2)/a^3 + Nobs*(vhigh^2/a^3)*ehigh/Fhigh;
	g = -g;
	% Hessian matrix and negative since minimizing
	H = 2*Nobs/a^2 - 3*sum(v.^2)/a^4 + Nobs*ehigh/Fhigh^2*...
		((vhigh^2/a^3)^2 - 3*vhigh^2/a^4*Fhigh);
	H = -H;	
elseif strcmpi(dist,'weibull')
	% log-likelihood function and negative since minimizing
	Fhigh = 1 - exp(-(vhigh/a)^b);
	L = sum(log(b) - b*log(a) + (b - 1)*log(v) - (v./a).^b - log(Fhigh));
	L = -L;
	% score vector and negative since minimizing
	ehigh = 1 - Fhigh;
	dFhighda = -b/a^(b + 1)*vhigh^b*ehigh;
	dFhighdb = (vhigh/a)^b*log(vhigh/a)*ehigh;
	g(1) = sum(-b/a + b/a^(b + 1)*v.^b - dFhighda/Fhigh);
	g(2) = sum(1/b - log(a) + log(v) - (v./a).^b.*log(v./a) - ...
		dFhighdb/Fhigh);
	g = -g;
	% Hessian matrix and negative since minimizing
	d2Fhighda2 = ehigh*(b*(b + 1)*vhigh^b/a^(b + 2) - ...
		(b*vhigh^b/a^(b + 1))^2);
	d2Fhighdb2 = ehigh*(vhigh/a)*(log(vhigh/a))^2*(1 - (vhigh/a)^b);
	d2Fhighdadb = -ehigh*(vhigh/a)^b*(1/a + (b/a)*log(vhigh/a) - ...
		b*vhigh^b/a^(b + 1)*log(vhigh/a));
	H(1,1) = sum(b/a^2 + b*(-b - 1)*v.^b*a^(-b - 2) - ...
		(((Fhigh - 1)*dFhighda^2 + ...
		(Fhigh - 1)*(b*(-b - 1)*vhigh^b*a^(-b - 2)))/Fhigh - ...
		(Fhigh - 1)^2*dFhighda^2/Fhigh^2));
	H(1,2) = sum(-1/a + v.^b*a^(-b - 1) + b*v.^b*a^(-b - 1).*log(v./a) - ...
		(((Fhigh - 1)*dFhighdb*dFhighda + (Fhigh - 1)* ...
		(vhigh^b*a^(-b - 1) + b/a*(vhigh/a)^b*log(vhigh/a)))/Fhigh - ...
		(Fhigh - 1)^2*dFhighda*dFhighdb/Fhigh^2));
	H(2,1) = H(1,2);
	H(2,2) = sum(-1/b^2 - (v./a).^b.*(log(v./a)).^2 - ...
		(((Fhigh - 1)*dFhighdb^2 + ...
		(Fhigh - 1)*(-(vhigh/a)^b*(log(vhigh/a))^2))/Fhigh - ...
		(Fhigh - 1)^2*dFhighdb^2/Fhigh^2));
	H = -H;
elseif strcmpi(dist,'exponential')
	% log-likelihood function and negative since minimizing
	Fhigh = 1 - exp(-a*vhigh);
	fhigh = vhigh*exp(-a*vhigh);
	L = sum(log(a) - a*v - log(Fhigh));
	L = -L;
	% score vector and negative since minimizing
	g = sum(1/a - v - fhigh/Fhigh);
	g = -g;
	% Hessian matrix and negative since minimizing
	H = Nobs*(-1/a^2 + vhigh*fhigh/(Fhigh^2));
	H = -H;
elseif strcmpi(dist,'power')
	% log-likelihood function and negative since minimizing
	L = sum(log(a) + (a - 1)*log(v));
	L = -L;
	% score vector and negative since minimizing
	g = sum(1/a + log(v));
	g = -g;
	% Hessian matrix and negative since minimizing
	H = -Nobs/a^2;
	H = -H;
end