function [Exprev] = ComputeExpRev(betaV,knotvecV,lambdahat,r)

%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% compute expected revenue using B-spline estimates of distribution of
% valuations

r = max(r,knotvecV(1));
[BasisVr] = BsplineBasis3(knotvecV,r);
FVr = BasisVr*betaV;

% compute probabilities of various N realizations, expected payoff to
% seller if only one bidder tenders offer that exceeds reserve
maxN = 100;
N = 1:1:maxN;
probN = poissgenpdf(N,lambdahat);
weight = probN;
reservesum = 0;
for n=1:maxN
	reservesum = reservesum + weight(n)*n*FVr.^(n - 1);
end
Exprevterm1 = r*(1 - FVr)*reservesum;

% integral term
vhigh = knotvecV(end);
Exprevterm2 = integral(@ApplicationIntegral,r,vhigh,'RelTol',1e-12,'AbsTol',1e-12);

Exprev = Exprevterm1 + Exprevterm2;


% this is the function that actually specifies the integral to be computed
	function intterm = ApplicationIntegral(v)
		
		% make sure input (evaluation) vector is a column vector
		alterout = 0;
		if iscolumn(v) == 0
			v = v';
			alterout = 1;
		end
		
		% evaluate distribution and density at v
		[BasisV,~,DerivV] = BsplineBasis3(knotvecV,v);
		FVv = BasisV*betaV;
		fVv = DerivV*betaV;
		
		% account for unknown number of bidders by forming appropriate
		% weighted sum 
		sumterm = 0;
		for i=1:maxN
			sumterm = sumterm + weight(i)*i*(i - 1)*FVv.^(i - 2);
		end
		
		intterm = v.*(1 - FVv).*fVv.*sumterm;
		if alterout == 1
			intterm = intterm';
		end
	end


end