function F2 = Fprime2(v,l,u,dist,a,b)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
%FPRIME2(v,l,u,dist,a,b) calculates the second derivative of the
%distribution dist, where dist can be 'lognormal', 'normal', 'Rayleigh',
%'normal', 'Weibull', 'exponential', or 'power'.
% 
%See also TLOGNCDF, TNORMCDF, TRAYLCDF, TLOGNPDF, TNORMPDF, TRAYLPDF, 
%TWBLPDF, TWBLCDF, PWRCDF, PWRPDF


if nargin<5
    error('Not enought input arguments for function G')
elseif  nargin<6 && (strcmpi(dist,'rayleigh')+strcmpi(dist,'power')+strcmpi(dist,'exponential')==0)
    error(strcat('Not enought input arguments for distribution ',dist))
elseif ischar(dist)==0
    display('You must specify a distribution over private values.  Type either normal, ');
    display('weibull, uniform, rayleigh, or power IN SINGLE QUOTES.');
    error('ERROR');
end

if strcmpi(dist,'exponential')
     dist = 'weibull';
     b = 1;
     a = 1/a;
elseif strcmpi(dist,'uniform')
    dist = 'power';
    a = 1;
    b = 1;
end

if strcmpi(dist,'lognormal')
    F2 = -tlognpdf(v,l,u,a,b).*(v.^-1).*( (b^2 + log(v) - a)/(b^2) );  
    
elseif strcmpi(dist,'normal')
    F2 = -tnormpdf(v,l,u,a,b).*( (v-a)/(b^2) );  
    
elseif strcmpi(dist,'rayleigh')
    F2 = traylpdf(v,l,u,a).*( v.^-1 - v/(b^2) );    
    
elseif strcmpi(dist,'weibull')
    F2 = twblpdf(v,l,u,a,b).*( (b-1)./v - b*(a^-b)*v.^(b-1) );   
    
elseif strcmpi(dist,'power')
    F2 = pwrpdf(v,l,u,a).*(a-1).*(v-l).^-1; 
end