function dist = G(v,N,l,u,dist,a,b)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
%G(v,N,l,u,dist,a,b) computes the distribution of the second-highest order 
%statistic from a random sample of size N following distribution dist with
%parameters a and b.  Here, dist can be 'lognormal', 'normal', 'Rayleigh',
%'normal', 'Weibull', 'exponential', or 'power'.  The other inputs l and u
%are truncation points that the user may specify for the underlying parent
%distribution.
% 
%See also TLOGNCDF, TNORMCDF, TRAYLCDF, TLOGNPDF, TNORMPDF, TRAYLPDF, 
%TWBLPDF, TWBLCDF, PWRCDF, PWRPDF


if nargin<6
    error('Not enought input arguments for function G')
elseif  nargin<7 && (strcmpi(dist,'rayleigh')+strcmpi(dist,'power')+strcmpi(dist,'exponential')==0)
    error(strcat('Not enought input arguments for distribution ',dist))
elseif ischar(dist)==0
    display('You must specify a distribution over private values.  Type either normal, ');
    display('weibull, uniform, rayleigh, or power IN SINGLE QUOTES.');
    error('ERROR');
end

if strcmpi(dist,'exponential')
     dist = 'weibull';
     b = 1;
     a = 1/a;
elseif strcmpi(dist,'uniform')
    dist = 'power';
    a = 1;
    b = 1;
end

if strcmpi(dist,'lognormal')
    dist = tlogncdf(v,l,u,a,b).^(N-1);  %CDF of the second highest order statistic
    
elseif strcmpi(dist,'normal')
    dist = tnormcdf(v,l,u,a,b).^(N-1);  %CDF of the second highest order statistic
    
elseif strcmpi(dist,'rayleigh')
    dist = traylcdf(v,l,u,a).^(N-1);    %CDF of the second highest order statistic
    
elseif strcmpi(dist,'weibull')
    dist = twblcdf(v,l,u,a,b).^(N-1);   %CDF of the second highest order statistic
    
elseif strcmpi(dist,'power')
    dist = pwrcdf(v,l,u,a).^(N-1);      %CDF of the second highest order statistic
end