function G3 = Gprime3(v,N,l,u,dist,a,b)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
%Gprime2(v,N,l,u,dist,a,b) computes the third derivative of the distribution
%over the second-highest order statistic from a random sample of size N following 
%distribution dist with parameters a and b.  Here, dist can be 'lognormal', 
%'normal', 'Rayleigh', 'normal', 'Weibull', 'exponential', or 'power'.  The other 
%inputs l and u are truncation points that the user may specify for the 
%underlying parent distribution.
% 
%See also TLOGNCDF, TNORMCDF, TRAYLCDF, TLOGNPDF, TNORMPDF, TRAYLPDF, 
%TWBLPDF, TWBLCDF, PWRCDF, PWRPDF


if nargin<6
    error('Not enought input arguments for function G')
elseif  nargin<7 && (strcmpi(dist,'rayleigh')+strcmpi(dist,'power')+strcmpi(dist,'exponential')==0)
    error(strcat('Not enought input arguments for distribution ',dist))
elseif ischar(dist)==0
    display('You must specify a distribution over private values.  Type either normal, ');
    display('weibull, uniform, rayleigh, or power IN SINGLE QUOTES.');
    error('ERROR');
end

if strcmpi(dist,'exponential')
     dist = 'weibull';
     b = 1;
     a = 1/a;
elseif strcmpi(dist,'uniform')
    dist = 'power';
    a = 1;
    b = 1;
end

if strcmpi(dist,'lognormal')  %Third derivative of the CDF of the second highest order statistic
    F  = tlogncdf(v,l,u,a,b);
    F1 = tlognpdf(v,l,u,a,b);
    G3 = (N-1)*(N-2)*(N-3)*(F.^(N-4)).*F1.^3 ...
           + 3*(N-1)*(N-2)*(F.^(N-3)).*F1.*Fprime2(v,l,u,'lognormal',a,b) ...
                 +   (N-1)*(F.^(N-2)).*Fprime3(v,l,u,'lognormal',a,b);  
    
elseif strcmpi(dist,'normal')  %Third derivative of the CDF of the second highest order statistic
    F  = tnormcdf(v,l,u,a,b);
    F1 = tnormpdf(v,l,u,a,b);
    G3 = (N-1)*(N-2)*(N-3)*(F.^(N-4)).*F1.^3 ...
           + 3*(N-1)*(N-2)*(F.^(N-3)).*F1.*Fprime2(v,l,u,'normal',a,b) ...
                 +   (N-1)*(F.^(N-2)).*Fprime3(v,l,u,'normal',a,b);
    
elseif strcmpi(dist,'rayleigh')  %Third derivative of the CDF of the second highest order statistic
    F  = traylcdf(v,l,u,a);
    F1 = traylpdf(v,l,u,a);
    G3 = (N-1)*(N-2)*(N-3)*(F.^(N-4)).*F1.^3 ...
           + 3*(N-1)*(N-2)*(F.^(N-3)).*F1.*Fprime2(v,l,u,'rayleigh',a) ...
                 +   (N-1)*(F.^(N-2)).*Fprime3(v,l,u,'rayleigh',a);
    
elseif strcmpi(dist,'weibull')  %Third derivative of the CDF of the second highest order statistic
    F  = twblcdf(v,l,u,a,b);
    F1 = twblpdf(v,l,u,a,b);
    G3 = (N-1)*(N-2)*(N-3)*(F.^(N-4)).*F1.^3 ...
           + 3*(N-1)*(N-2)*(F.^(N-3)).*F1.*Fprime2(v,l,u,'weibull',a,b) ...
                 +   (N-1)*(F.^(N-2)).*Fprime3(v,l,u,'weibull',a,b);
    
elseif strcmpi(dist,'power')  %Third derivative of the CDF of the second highest order statistic
    F  = pwrcdf(v,l,u,a);
    F1 = pwrpdf(v,l,u,a);
    G3 = (N-1)*(N-2)*(N-3)*(F.^(N-4)).*F1.^3 ...
           + 3*(N-1)*(N-2)*(F.^(N-3)).*F1.*Fprime2(v,l,u,'power',a) ...
                 +   (N-1)*(F.^(N-2)).*Fprime3(v,l,u,'power',a);
end