function [Exprev,hypIncExprev,hypDecExprev,optimalr] = ...
	RevenueCalculations(betaVPE,knotvecV,lambdaPE)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% compute expected revenue for various instances

% compute expected revenues for a grid of reserve prices
ngrid = 1000;
rvect = linspace(knotvecV(1),knotvecV(end),ngrid); 

% optimal reserve price and maximum expected revenue
if nargout == 4
	resguess = knotvecV(1)*1.25;
	%optimalr = fzero('optreserveBspline',resguess,[],betaVPE,knotvecV);
	optimalr = fminbnd('optreserveBspline',200,400,[],betaVPE,knotvecV);
	maxExprev = ComputeExpRev(betaVPE,knotvecV,lambdaPE,optimalr);
end

for i=1:ngrid
	Exprev(i) = ComputeExpRev(betaVPE,knotvecV,lambdaPE,rvect(i));
end

meanN = lambdaPE(1)/(1 - lambdaPE(2));
varN = meanN/(1 - lambdaPE(2))^2;

if nargout >= 2
	hypInclambda = lambdaPE;
	x = 1;
	hypInclambda(2) = 1 - sqrt((meanN + x)/varN);
	hypInclambda(1) = (1 - hypInclambda(2))^3*varN;
	for i=1:ngrid
		hypIncExprev(i) = ComputeExpRev(betaVPE,knotvecV,hypInclambda,rvect(i));
	end
end

if nargout >= 3
	hypDecclambda = lambdaPE;
	y = -1;
	hypDecclambda(2) = 1 - sqrt((meanN + y)/varN);
	hypDecclambda(1) = (1 - hypDecclambda(2))^3*varN;
	for i=1:ngrid
		hypDecExprev(i) = ComputeExpRev(betaVPE,knotvecV,hypDecclambda,rvect(i));
	end
end