%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% compare bidding at EA and SP auctions

clear all
clc
% parameters for figure properties
lwidth = 2;
fsize = 14;
set(0,'defaulttextinterpreter','latex','Defaulttextfontsize',fsize);

% distribution of interest
vlow = 0;
vhigh = 1;
% if exponential think of param1 = lambda (probably want to be greater than
% 1...although mean is different given truncation); if rayleigh probably
% want mean to be less than sqrt(pi)/2 so that mean is less than one.
% weibull is an exponential if param2 = 1 and is a rayleigh if param2 = 2.
dist = 'rayleigh';
param1 = 1;
param2 = 2;
pvect = param1;
if strcmpi(dist,'weibull')
	pvect = [pvect param2];
end
delta = 0.1;
N = 3;

% solve for the inverse-bid function given the distribution
rkorder = 6;
[vals,bids] = EAauction_compare(N,vhigh,delta,dist,param1,param2,rkorder);
if sum(vals - bids < 0) > 0
	warning('Bid function is not rational: some bids exceed valuations\n')
	return
end

v = linspace(vlow,vhigh,10000);
b_ea = spline(vals,bids,v);
if sum(v - b_ea < 0) > 0
	warning('Bid function is not rational: some bids exceed valuations\n')
	return
end

[pv,bidfpa] = FPauction(N,vhigh,dist,param1,param2);
b_fpa = spline(pv,bidfpa,v);
if sum(v - b_fpa < 0) > 0
	warning('Bid function is not rational: some bids exceed valuations\n')
	return
end


figure1 = figure;
set(gcf,'DefaultLineLineWidth',lwidth)
set(gca,'FontSize',fsize)
hold all
box on
plot(v,v,v,b_ea,v,b_fpa)
xlabel('$\mathbf{v}$')
ylabel('$\mathbf{b}$')
xlim([vlow vhigh])
annotation(figure1,'textbox',[0.793 0.5668 0.06608 0.05941],...
	'String',{'$\mathbf{\beta_{FP}(v)}$'},...
	'FitBoxToText','off',...
	'LineStyle','none','Interpreter', 'latex');
annotation(figure1,'textbox',[0.8203 0.7079 0.06608 0.05941],...
	'String',{'$\mathbf{\beta_{EA}(v)}$'},...
	'FitBoxToText','off',...
	'LineStyle','none','Interpreter', 'latex');
annotation(figure1,'textbox',[0.7136 0.8098 0.06608 0.05941],...
	'String',{'$\mathbf{\beta_{SP}(v)}$'},...
	'FitBoxToText','off',...
	'LineStyle','none','Interpreter', 'latex');


print('-depsc','bfcncompare.eps')