function integrand = expected_bferr(v,dist,vlow,vhigh,a,b, ...
	pvsoln,bidsoln,type)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% compute expected error from assuming second-price auction when really EA
% auction exists

% evaluate density at valuation of interest
if strcmpi(dist,'weibull')
	F = 1 - exp(-(v./a).^b);
	f = (b/a).*(v./a).^(b - 1).*exp(-(v./a).^b);
	Ftr = 1 - exp(-(vhigh/a)^b);
	Fv = F./Ftr;
	pdf = f./Ftr;
elseif strcmpi(dist,'power')
	Fv = v.^a;
	pdf = a*v.^(a - 1);
elseif strcmpi(dist,'exponential')
	F = 1 - exp(-a*v);
	f = a*exp(-a*v);
	Ftr = 1 - exp(-a*vhigh);
	Fv = F./Ftr;
	pdf = f./Ftr;
elseif strcmpi(dist,'rayleigh')
	F = 1 - exp(-(v.^2)./(2*a^2));
	f = v./a^2.*exp(-(v.^2)./(2*a^2));
	Ftr = 1 - exp(-(vhigh^2)./(2*a^2));
	Fv = F./Ftr;
	pdf = f./Ftr;
end
% evaluate EA bid function at valuation of interest which involves
% piecewise cubic Hermite interpolation over the solved bid function---this
% preserves monotonicity and shape
ea_bf = interp1(pvsoln,bidsoln,v,'pchip');

% compute error at that valuation
if strcmpi(type,'abs')
	errterm = v - ea_bf;
elseif strcmpi(type,'rel')
	errterm = (v - ea_bf)./v;
end

% construct integrand
integrand = errterm.*pdf;