function optcond = optreserve(r,dist,vhigh,param)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% compute optimal reserve price for given instance

[nr,nc] = size(param);
a = param(1);
if (nr > 1) || (nc > 1)
	b = param(2);
end

if strcmpi(dist,'rayleigh')
	F = 1 - exp(-r.^2/(2*a^2));
	f = r/a^2*exp(-r.^2/(2*a^2));
	Ftr = 1 - exp(-vhigh^2/(2*a^2));
	Fv = F./Ftr;
	fv = f./Ftr;
elseif strcmpi(dist,'weibull')
	F = 1 - exp(-(r./a).^b);
	f = (b/a)*(r./a).^(b - 1).*exp(-(r./a).^b);
	Ftr = 1 - exp(-(vhigh/a)^b);
	Fv = F./Ftr;
	fv = f./Ftr;
elseif strcmpi(dist,'exponential')
	F = 1 - exp(-a*r);
	f = a*exp(-a*r);
	Ftr = 1 - exp(-a*vhigh);
	Fv = F./Ftr;
	fv = f./Ftr;
elseif strcmpi(dist,'power')
	Fv = r.^a;
	fv = a*r.^(a - 1);
end

haz = fv./(1 - Fv);
optcond = r - 1./haz;
%plot(r,optcond,r,zeros(size(r)));