%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%

clear all 
close all
clc

%matlabpool open torque 190
matlabpool open
for k=1:100  %%%%To get around memory constraints, we will run the thing 10 times each with 10^8 simulations each time for each Ntilde.
    tic
    if k<11
        rng(pi*1000000*k)  %For each of the runs 1-10 we reset the seed to a different number...
    elseif k<21
        rng(exp(1)*1000000*(k-10))  %For each of the runs 11-20 we reset the seed to a different number...
    elseif k<31
        rng(sqrt(2)*1000000*(k-20))  %For each of the runs 21-30 we reset the seed to a different number...
    elseif k<41
        rng(sqrt(3)*1000000*(k-30))  %For each of the runs 31-40 we reset the seed to a different number...
    elseif k<51
            rng(sqrt(5)*1000000*(k-40))  %For each of the runs 41-50 we reset the seed to a different number...
    elseif k<61
            rng(sqrt(7)*1000000*(k-50))  %For each of the runs 51-60 we reset the seed to a different number...
    elseif k<71
            rng(sqrt(11)*1000000*(k-60))  %For each of the runs 61-70 we reset the seed to a different number...
    elseif k<81
            rng(sqrt(13)*1000000*(k-70))  %For each of the runs 71-80 we reset the seed to a different number...
    elseif k<91
            rng(sqrt(17)*1000000*(k-80))  %For each of the runs 81-90 we reset the seed to a different number...
    else
            rng(sqrt(19)*1000000*(k-90))  %For each of the runs 91-100 we reset the seed to a different number...
    end
    
    display('GENERATING A LIST OF PARTICIPATION LEVELS...')
    display('')
    Ktilde = 100;  %%%%This is the total number of values for Ntilde in the simulation
    SimInd = 100000000;  %%%%This is the total number of simulations for each value of Ntilde
    SimTot = SimInd*(Ktilde); %%%%
    Ntilde = ones(Ktilde,SimInd); %Ntilde is the actual number of bidders
    N      = ones(SimTot,1); %N is the observed number of unique bidders.  It's 1 if Ntilde==1
    
    
    display('NOW SHUFFLING THE LIST...')
    display('')
    for i=2:Ktilde
        Ntilde(i,:) = i;
    end
    Ntilde = reshape(Ntilde,SimTot,1);
    
    
    display('NOW OPENING A WORKER POOL TO BEGIN SIMULATION...')
    display('')
    parfor i=1:SimTot
        if Ntilde(i)==2
            N(i) = 2; %N is always at least 2 when Ntilde is >=2
        elseif Ntilde(i)>2
            N(i) = 2; %N is always at least 2 when Ntilde is >=2
            auction  = rand(Ntilde(i),1);  %%%%The first column represents the quantile rank of each bid and the second column holds a random ordering variable
            %             observed = zeros(Ntilde(i),1);
            P        = min(auction(1:2));
            Pcap     = max(auction(1:2));
            %             observed(1:2) = 1;
            for j=3:Ntilde(i);
                if auction(j,1)>P
                    if auction(j,1)> Pcap;
                        P           = Pcap;
                        Pcap        = auction(j,1);
                        %                     observed(j) = 1;
                        N(i) = N(i) + 1;
                    else
                        P           = auction(j,1);
                        %                     observed(j) = 1;
                        N(i) = N(i) + 1;
                    end
                end
            end
            %             N(i) = sum(observed);
        end
    end
    
    
    SIMULATING_TIME_IN_HRS = toc/3600
    tic
        
    
    %%
    ProbNtildegN = zeros(Ktilde);
    ProbNtildegN(1,1) = 1;
    ProbNtildegN(2,2) = 1;
    N=reshape(N,Ktilde,SimInd);
    
    for i=3:Ktilde
        display(strcat('Now computing probabilities conditional on Ntilde=',num2str(i),'... (for k=',num2str(k),')'))
        display('')
        Ntemp = N(i,:);
        parfor j=1:i
            ProbNtildegN(i,j) = mean(Ntemp==j);
        end
    end
    command = strcat('ProbNtildegN',num2str(k),'=ProbNtildegN;');
    eval(command)
    command = strcat('save(''ProbNtildegN',num2str(k),''',''ProbNtildegN',num2str(k),''')');
    eval(command)
    
    clear Ktilde N Ntemp Ntilde P Pcap ProbNtildegN SimInd SimTot auction command i j

    PROBABILITY_COMP_TIME_IN_HRS = toc/3600
end

matlabpool close


%%
donesofar = 100;
ProbNtildegN = zeros(100,100);
for k=1:donesofar
    command = strcat('load(''ProbNtildegN',num2str(k),'.mat'');');
    eval(command)
    command = strcat('ProbNtildegN = ProbNtildegN + ProbNtildegN',num2str(k),';');
    eval(command)
end

ProbNtildegN = ProbNtildegN/donesofar;

save('ProbNtildegN','ProbNtildegN');


display('')
display('FINAL SUMMARY OF OUTPUT (100 blocks of 10^8):')
test = reshape(ProbNtildegN1,1,100*100);
for i=2:donesofar;
    command = strcat('test = [test; reshape(ProbNtildegN',num2str(i),',1,100*100)];'); 
    eval(command); 
end

TEST=std(test);
MEAN_STD_ERROR_10_2the_8th            = mean(TEST(test(1,:)>0)) 
MAX_STD_ERROR_10_2the_8th             = max(TEST(test(1,:)>0)) 
PROJECTED_MEAN_STD_ERROR_10_2the_10th = mean(TEST(test(1,:)>0))/sqrt(donesofar)  
PROJECTED_MAX_STD_ERROR_10_2the_10th  = max(TEST(test(1,:)>0))/sqrt(donesofar) 


display('')
display('FINAL SUMMARY OF OUTPUT (10 blocks of 10^9):')
test = reshape(ProbNtildegN1,1,100*100);
for i=2:10;
    command = strcat('test = test + reshape(ProbNtildegN',num2str(i),',1,100*100);'); 
    eval(command); 
end
test = test/10;

for i=2:10
    temp = [];
    command = strcat('temp = reshape(ProbNtildegN',num2str((i-1)*10+1),',1,100*100);');
    eval(command);
    for j=2:10
        command = strcat('temp = temp + reshape(ProbNtildegN',num2str((i-1)*10+j),',1,100*100);');
        eval(command);
    end
    temp = temp/10;
    test = [test; temp];
end

TEST=std(test);
MEAN_STD_ERROR_10_2the_9th            = mean(TEST(test(1,:)>0))  
MAX_STD_ERROR_10_2the_9th             = max(TEST(test(1,:)>0))  
PROJECTED_MEAN_STD_ERROR_10_2the_10th = mean(TEST(test(1,:)>0))/sqrt(10)  
PROJECTED_MAX_STD_ERROR_10_2the_10th  = max(TEST(test(1,:)>0))/sqrt(10) 


